@@ -22,8 +22,18 @@
  */
 #include "newgamedlg.h"
 
+#include "../players/ai/becai/becai_gui.h"
+#include "../players/ai/becai/becai.h"
+#include "../players/ai/default/weak_gui.h"
+#include "../players/ai/default/normal_gui.h"
+#include "../players/ai/default/hard_gui.h"
+#include "../players/ai/example/example_gui.h"
 #include "../players/computerplayer.h"
+#include "../players/player_gui.h"
 #include "../players/localplayer.h"
+#include "../players/localplayer_gui.h"
+#include "../players/spectatorplayer.h"
+#include "../players/spectatorplayer_gui.h"
 #include "../game.h"
 #include <kconfig.h>
 #include <klocale.h>
@@ -36,6 +46,7 @@
 #include <KComboBox>
 #include <QHeaderView>
 #include <QItemDelegate>
+#include <QList>
 #include <QDebug>
 #include <QLinkedList>
 #include <QPair>
@@ -61,9 +72,14 @@ class playersListModel : public QAbstractTableModel
 {
     typedef QPair<QColor, QString> PlayerId;
     QLinkedList<PlayerId> m_availablePlayerId;
+    NewGameDlg *m_newGameDlg;
+
 public:
-    playersListModel(QObject *parent, Game *game)
-        : QAbstractTableModel(parent), m_game(game)
+    playersListModel(NewGameDlg *newGameDlg, Game *game, const QList<PlayerGui*> &selectablePlayer) :
+        QAbstractTableModel(newGameDlg),
+        m_newGameDlg(newGameDlg),
+        m_game(game),
+        m_selectablePlayer(selectablePlayer)
     {
         for(int a = 0; a < MAX_PLAYERS; ++a)
         {
@@ -102,21 +118,7 @@ public:
                 }
                 else if (column == 1)
                 {
-                    LocalPlayer *lPlayer = qobject_cast<LocalPlayer*>(player);
-                    ComputerPlayer *cPlayer = qobject_cast<ComputerPlayer*>(player);
-                    if (lPlayer)
-                    {
-                        return i18nc("A human player", "Human");
-                    }
-                    else
-                    {
-                        switch (cPlayer->aiLevel())
-                        {
-                        case ComputerPlayer::Weak: return i18n("Computer Weak"); break;
-                        case ComputerPlayer::Normal: return i18n("Computer Normal"); break;
-                        case ComputerPlayer::Hard: return i18n("Computer Hard"); break;
-                        }
-                    }
+                    return player->guiName();
                 }
             }
         }
@@ -144,49 +146,34 @@ public:
             Player *player = m_players.at(row);
             if (column == 0)
             {
+
+                // The player name changed.
+
                 player->setName(value.toString());
                 result = true;
             }
             else if (column == 1)
             {
+
+                // The player controller changed.
+
                 QString text = value.toString();
-                ComputerPlayer *cPlayer = qobject_cast<ComputerPlayer*>(player);
-                if (cPlayer && (text == i18n("Computer Weak")))
-                    cPlayer->setAiLevel(ComputerPlayer::Weak);
-                else if (cPlayer && (text == i18n("Computer Normal")))
-                    cPlayer->setAiLevel(ComputerPlayer::Normal);
-                else if (cPlayer && (text == i18n("Computer Hard")))
-                    cPlayer->setAiLevel(ComputerPlayer::Hard);
-                else if (cPlayer && (text == i18nc("A human player", "Human")))
-                {
-                    LocalPlayer *newPlayer = new LocalPlayer(player->game(), cPlayer->name(), cPlayer->color());
+
+                Player *newPlayer = getNewPlayerByGuiName(text, player->name(), player->color());
+
+                if (newPlayer) {
                     m_players[row] = newPlayer;
                     m_game->map()->turnOverPlayerPlanets(player, newPlayer);
-                    cPlayer->deleteLater();
-                    player = newPlayer;
-                    cPlayer = 0;
-                }
-                else if (text != i18nc("A human player", "Human"))
-                {
-                    ComputerPlayer::AiLevel lvl = ComputerPlayer::Weak;
-                    if (text == i18n("Computer Weak"))
-                        lvl = ComputerPlayer::Weak;
-                    else if (text == i18n("Computer Normal"))
-                        lvl = ComputerPlayer::Normal;
-                    else if (text == i18n("Computer Hard"))
-                        lvl = ComputerPlayer::Hard;
-                    cPlayer = new ComputerPlayer(player->game(), player->name(), player->color(), lvl);
-                    m_players[row] = cPlayer;
-                    m_game->map()->turnOverPlayerPlanets(player, cPlayer);
                     player->deleteLater();
-                    player = cPlayer;
+
+                    result = true;
                 }
-                result = true;
             }
         }
 
         if (result) {
             m_game->setPlayers(m_players);
+            m_newGameDlg->updateButtonOk();
             emit dataChanged(index, index);
         }
 
@@ -197,14 +184,28 @@ public:
     {
         Player *player = NULL;
         int players = m_players.count();
-        if (players < MAX_PLAYERS)
-        {
+
+        if ((!m_availablePlayerId.empty()) && (!m_selectablePlayer.empty())) {
             beginInsertRows(QModelIndex(), players, players);
 
-            player = new LocalPlayer(m_game, m_availablePlayerId.front().second, m_availablePlayerId.front().first);
-            m_availablePlayerId.pop_front();
-            m_players.append(player);
-            m_game->setPlayers(m_players);
+            // Get the player controller of the last added player. If no player
+            // has been added yet, use the very first registered player
+            // controller as default.
+
+            QString guiName = m_selectablePlayer.front()->guiName();
+
+            if (!m_players.isEmpty()) {
+                guiName = m_players.last()->guiName();
+            }
+
+            player = getNewPlayerByGuiName(guiName, m_availablePlayerId.front().second, m_availablePlayerId.front().first);
+
+            if (player) {
+                m_availablePlayerId.pop_front();
+                m_players.append(player);
+                m_game->setPlayers(m_players);
+                m_newGameDlg->updateButtonOk();
+            }
 
             endInsertRows();
         }
@@ -226,30 +227,54 @@ public:
             m_availablePlayerId.push_back(PlayerId(player->color(), player->name()));
             m_players.removeAt(row);
             m_game->setPlayers(m_players);
+            m_newGameDlg->updateButtonOk();
             endRemoveRows();
         }
         return player;
     }
 
-    bool hasHumans() const
+private:
+
+    /**
+     * Search the list of registered player controllers and return either a new
+     * instance of that controller or NULL if it cannot be found.
+     *
+     * @note The GUI name of the player instance gets set.
+     */
+
+    Player *getNewPlayerByGuiName(const QString &guiName, const QString &playerName, const QColor &color) const
     {
-        foreach (Player *player, m_players)
-        {
-            if (!player->isAiPlayer())
-                return true;
+        Player *newPlayer = 0;
+
+        foreach (PlayerGui *playerGui, m_selectablePlayer) {
+            if (guiName == playerGui->guiName()) {
+                newPlayer = playerGui->createInstance(m_game, playerName, color);
+                newPlayer->setGuiName(guiName);
+                break;
+            }
         }
-        return false;
+
+        return newPlayer;
     }
 
 private:
     Game *m_game;
     QList<Player *> m_players;
+    QList<PlayerGui*> m_selectablePlayer;
 };
 
+
+/**
+ * This class handles the list of selectable player controllers.
+ * This includes the local player and all enabled AI players.
+ */
+
 class playersListDelegate : public QItemDelegate
 {
 public:
-    playersListDelegate(QObject *parent) : QItemDelegate(parent)
+    playersListDelegate(QObject *parent, const QList<PlayerGui*> &selectablePlayer) :
+        QItemDelegate(parent),
+        m_selectablePlayer(selectablePlayer)
     {
     }
 
@@ -265,10 +290,10 @@ public:
     {
         if (index.column() != 0) {
             KComboBox *cbox = static_cast<KComboBox*>(editor);
-            cbox->addItem(i18nc("A human player", "Human"));
-            cbox->addItem(i18n("Computer Weak"));
-            cbox->addItem(i18n("Computer Normal"));
-            cbox->addItem(i18n("Computer Hard"));
+
+            foreach (PlayerGui* playerGui, m_selectablePlayer) {
+                cbox->addItem(playerGui->guiName());
+            }
 
             cbox->setCurrentIndex( cbox->findText(index.data( Qt::DisplayRole).toString()) );
         } else {
@@ -289,6 +314,9 @@ public:
             model->setData(index, lineEdit->text(), Qt::EditRole);
         }
     }
+
+private:
+    QList<PlayerGui*> m_selectablePlayer;
 };
 
 NewGameDlg::NewGameDlg( QWidget *parent, Game *game)
@@ -301,18 +329,26 @@ NewGameDlg::NewGameDlg( QWidget *parent, Game *game)
     setDefaultButton(KDialog::NoDefault);
     showButtonSeparator(true);
 
+    m_selectablePlayer.push_back(new LocalPlayerGui());
+    m_selectablePlayer.push_back(new SpectatorPlayerGui());
+    m_selectablePlayer.push_back(new AiDefaultWeakGui());
+    m_selectablePlayer.push_back(new AiDefaultNormalGui());
+    m_selectablePlayer.push_back(new AiDefaultHardGui());
+    // m_selectablePlayer.push_back(new AiExampleGui());
+    m_selectablePlayer.push_back(new AiBecaiGui());
+
     m_w = new NewGameDlgUI(this);
     m_w->map->setMap(m_game->map());
 
     connect(m_w->map, SIGNAL(sectorSelected(Coordinate)),
             this, SLOT(slotUpdateSelection(Coordinate)));
 
-    playersListModel *model = new playersListModel(this, m_game);
+    playersListModel *model = new playersListModel(this, m_game, m_selectablePlayer);
 
     m_w->playerList->setModel(model);
-    m_w->playerList->setItemDelegate(new playersListDelegate(this));
-    m_w->playerList->header()->setResizeMode(QHeaderView::Stretch);
-    
+    m_w->playerList->setItemDelegate(new playersListDelegate(this, m_selectablePlayer));
+    m_w->playerList->header()->setResizeMode(QHeaderView::ResizeToContents);
+
     connect(m_w->neutralPlanetsSB, SIGNAL(valueChanged(int)), this, SLOT(slotUpdateNeutrals(int)));
     connect(m_w->widthSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
     connect(m_w->heightSB, SIGNAL(valueChanged(int)), this, SLOT(slotNewMap()));
@@ -328,6 +364,13 @@ NewGameDlg::NewGameDlg( QWidget *parent, Game *game)
 
     setMainWidget(m_w);
     slotNewMap();
+
+    updateButtonOk();
+}
+
+NewGameDlg::~NewGameDlg()
+{
+    qDeleteAll(m_selectablePlayer);
 }
 
 void
@@ -341,6 +384,7 @@ NewGameDlg::updateOwnerCB()
     m_w->OwnerCB->addItem(i18n("neutral"));
 }
 
+
 void
 NewGameDlg::init()
 {
@@ -383,8 +427,9 @@ NewGameDlg::init()
         QString playerType = config.readEntry(keyType,QString());
 
         model->addPlayer();
-        if (!playerName.isEmpty())
+        if (!playerName.isEmpty()) {
             model->setData(model->index(i, 0), playerName, Qt::EditRole);
+        }
         model->setData(model->index(i, 1), playerType, Qt::EditRole);
     }
     updateOwnerCB();
@@ -396,11 +441,11 @@ NewGameDlg::slotAddPlayer()
     playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
 
     Player *player = model->addPlayer();
-    if (player)
+    if (player) {
         m_game->map()->addPlayerPlanetSomewhere(player);
+    }
 
-    enableButtonOk(model->rowCount() > 1);
-
+    updateButtonOk();
     updateOwnerCB();
 }
 
@@ -415,81 +460,76 @@ NewGameDlg::slotRemovePlayer()
         delete player;
     }
 
-    enableButtonOk(model->rowCount() > 1);
-
+    updateButtonOk();
     updateOwnerCB();
 }
 
-void
-NewGameDlg::slotOk()
-{
-    playersListModel *model = static_cast<playersListModel*>(m_w->playerList->model());
-    bool hasHumans = model->hasHumans();
-
-    if (!hasHumans) {
-        KMessageBox::information(this,
-                                 i18n("The game is much more fun when you add a human player!"));
-        return;
-    }
-
-    KDialog::accept();
-}
 
 void
 NewGameDlg::slotUpdateSelection(const Coordinate &coord)
 {
-    Sector *selected = m_game->map()->sector(coord);
-    if (!selected) {
+    Sector *sector = m_game->map()->sector(coord);
+    if (!sector) {
         m_w->OwnerCB->setEnabled(false);
         m_w->KillPercentageSB->setEnabled(false);
         m_w->ProductionSB->setEnabled(false);
         return;
     }
-    bool planet = selected->hasPlanet();
+    bool hasPlanet = sector->hasPlanet();
 
-    m_w->OwnerCB->setEnabled(planet);
-    m_w->KillPercentageSB->setEnabled(planet);
-    m_w->ProductionSB->setEnabled(planet);
+    m_w->OwnerCB->setEnabled(hasPlanet);
+    m_w->KillPercentageSB->setEnabled(hasPlanet);
+    m_w->ProductionSB->setEnabled(hasPlanet);
 
     disconnect(m_w->OwnerCB, SIGNAL(currentIndexChanged(int)), this, SLOT(slotNewOwner(int)));
 
-    if(planet) {
-        Planet *p = selected->planet();
-        int f = m_w->OwnerCB->findText(p->player()->name());
-        if(f>=0)
+    if (hasPlanet) {
+        Planet *planet = sector->planet();
+        int f = m_w->OwnerCB->findText(planet->player()->name());
+        if (f >= 0) {
             m_w->OwnerCB->setCurrentIndex(f);
-        else
+        }
+        else {
             m_w->OwnerCB->setCurrentIndex(m_w->OwnerCB->count()-1);
+        }
 
-
-        m_w->KillPercentageSB->setValue(p->killPercentage());
-        m_w->ProductionSB->setValue(p->production());
-    } else
+        m_w->KillPercentageSB->setValue(planet->killPercentage());
+        m_w->ProductionSB->setValue(planet->production());
+    }
+    else {
         m_w->OwnerCB->setCurrentIndex(0);
+    }
 
     connect(m_w->OwnerCB, SIGNAL(currentIndexChanged(int)), this, SLOT(slotNewOwner(int)));
 }
+
+
 void
 NewGameDlg::slotNewOwner(int index)
 {
     if (!m_w->map->hasSelection())
         return;
-    Sector *selected = m_game->map()->sector(m_w->map->selection());
-    if(selected->hasPlanet())
-        delete selected->planet();
+    Sector *sector = m_game->map()->sector(m_w->map->selection());
+    if (sector->hasPlanet()) {
+        delete sector->planet();
+    }
 
-    if(index > 0)
-        m_game->map()->addPlanet(selected,
+    if (index > 0) {
+        m_game->map()->addPlanet(sector,
                                  index - 1 < m_game->players().count()
                                  ? m_game->players()[index-1]
                                  :  m_neutral,
                                  m_w->ProductionSB->value(),
                                  m_w->KillPercentageSB->value());
+    }
 
     m_w->KillPercentageSB->setEnabled(index);
     m_w->ProductionSB->setEnabled(index);
+
+    updateButtonOk();
 }
 
+
 void
 NewGameDlg::slotNewKillPercentage(double value)
 {
@@ -498,6 +538,7 @@ NewGameDlg::slotNewKillPercentage(double value)
         selected->planet()->setKillPercentage(value);
 }
 
+
 void
 NewGameDlg::slotNewProduction(int value)
 {
@@ -543,6 +584,7 @@ NewGameDlg::save()
     config.sync();
 }
 
+
 void
 NewGameDlg::slotNewMap()
 {
@@ -563,8 +605,11 @@ NewGameDlg::slotNewMap()
     m_game->map()->populateMap(m_game->players(), m_neutral, m_w->neutralPlanetsSB->value());
 
     slotUpdateSelection(m_w->map->selection());
+
+    updateButtonOk();
 }
 
+
 void
 NewGameDlg::slotUpdateNeutrals(int count)
 {
@@ -577,3 +622,47 @@ NewGameDlg::slotUpdateNeutrals(int count)
     if(m_w->map->hasSelection())
         slotUpdateSelection(m_w->map->selection());
 }
+
+
+/**
+ * Enable the "ok" button if the game settings are sane.
+ */
+
+void
+NewGameDlg::updateButtonOk()
+{
+    int nonSpectatorCount = 0;
+    bool isSaneConfiguration = true;
+
+    foreach (Player *player, m_game->players()) {
+        if (player->isSpectator()) {
+
+            // Spectator player controllers can be completely ignored here as
+            // they do not count as active players and thus there is no need for
+            // them to own at least one planet.
+
+            continue;
+        }
+
+        ++nonSpectatorCount;
+
+        bool foundPlanet = false;
+
+        foreach (Planet *planet, m_game->planets()) {
+            if (planet->player() == player) {
+                foundPlanet = true;
+                break;
+            }
+        }
+
+        if (!foundPlanet) {
+            isSaneConfiguration = false;
+            break;
+        }
+    }
+
+    enableButtonOk(
+        (nonSpectatorCount >= 2) &&
+        isSaneConfiguration
+    );
+}
